==> open-xchange-documents-backend_7.8.2.orig.tar.bz2: open-xchange-documents-backend-7.8.2/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/BodyHandler.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.DrawingType;
import com.openexchange.office.odf.ElementNSWriter;
import com.openexchange.office.odf.SaxContextHandler;
import com.openexchange.office.odf.UnknownContentHandler;
import com.openexchange.office.odt.dom.components.Component;
import com.openexchange.office.odt.dom.components.ParagraphComponent;
import com.openexchange.office.odt.dom.components.RootComponent;

public class BodyHandler extends SaxContextHandler {

	final Body body;
	final private DLList<Object> pageAnchoredComponents = new DLList<Object>();

    public BodyHandler(SaxContextHandler parentContext, Body body)
    	throws SAXException {

    	super(parentContext);
    	this.body = body;
    }

    @Override
    public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName)
    	throws SAXException {

    	final SaxContextHandler textHandler = TextContentHelper.startElement(this, body, attributes, uri, localName, qName);
    	if(textHandler!=null) {
    		return textHandler;
    	}
    	if(qName.equals("draw:frame")) {
    		final DrawFrame drawFrame = new DrawFrame(attributes);
    		pageAnchoredComponents.add(drawFrame);
    		return new DrawFrameHandler(this, drawFrame);
    	}
    	final DrawingType drawingType = ShapeHelper.getDrawingType(qName);
    	if(drawingType!=null) {
    		return ShapeHelper.startElement(this, drawingType, pageAnchoredComponents, attributes, uri, localName, qName);
    	}
		final ElementNSWriter element = new ElementNSWriter(getFileDom(), attributes, uri, qName);
		body.getContent().add(element);
		return new UnknownContentHandler(this, element);
    }

	@Override
	public void endContext(String qName, String characters) {
		super.endContext(qName, characters);

		if(qName.equals("office:text")) {
			if(!pageAnchoredComponents.isEmpty()) {
				// move these objects into the first paragraph...
				final ParagraphComponent paragraphComponent = getFirstParagraphComponent(new RootComponent(body).getChildComponent(0));
				Paragraph paragraph;
				if(paragraphComponent!=null) {
					paragraph = (Paragraph)paragraphComponent.getObject();
				}
				else {
					paragraph = new Paragraph(null);
					body.getContent().add(paragraph);
				}
				final TextSpan textSpan = new TextSpan();
				paragraph.getContent().addFirst(textSpan);
				textSpan.setContent(pageAnchoredComponents);
			}
		}
	}

	private ParagraphComponent getFirstParagraphComponent(Component component) {
		while(component!=null) {
			if(component instanceof ParagraphComponent) {
				return (ParagraphComponent)component;
			}
			final ParagraphComponent childParagraphComponent = getFirstParagraphComponent(component);
			if(childParagraphComponent!=null) {
				return childParagraphComponent;
			}
			component = component.getNextComponent();
		}
		return null;
	}
}

==> open-xchange-documents-backend_7.8.3.orig.tar.bz2: open-xchange-documents-backend-7.8.3/com.openexchange.office.odf/src/com/openexchange/office/odt/dom/BodyHandler.java <==
/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

/**
 * @author sven.jacobi@open-xchange.com
 */

package com.openexchange.office.odt.dom;

import java.util.Stack;

import org.xml.sax.Attributes;
import org.xml.sax.SAXException;

import com.openexchange.office.odf.DLList;
import com.openexchange.office.odf.DrawingType;
import com.openexchange.office.odf.ElementNSWriter;
import com.openexchange.office.odf.SaxContextHandler;
import com.openexchange.office.odf.UnknownContentHandler;
import com.openexchange.office.odt.dom.components.Component;
import com.openexchange.office.odt.dom.components.ParagraphComponent;
import com.openexchange.office.odt.dom.components.RootComponent;

public class BodyHandler extends SaxContextHandler {

	final Body body;
	final private DLList<Object> pageAnchoredComponents = new DLList<Object>();
	private Stack<SectionStartMarker> sectionStack = null;

    public BodyHandler(SaxContextHandler parentContext, Body body)
    	throws SAXException {

    	super(parentContext);
    	this.body = body;
    }

    @Override
    public SaxContextHandler startElement(Attributes attributes, String uri, String localName, String qName)
    	throws SAXException {

    	final SaxContextHandler textHandler = TextContentHelper.startElement(this, body, attributes, uri, localName, qName);
    	if(textHandler!=null) {
    		return textHandler;
    	}
    	if(qName.equals("draw:frame")) {
    		final DrawFrame drawFrame = new DrawFrame(attributes);
    		pageAnchoredComponents.add(drawFrame);
    		return new DrawFrameHandler(this, drawFrame);
    	}
    	final DrawingType drawingType = ShapeHelper.getDrawingType(qName);
    	if(drawingType!=null) {
    		return ShapeHelper.startElement(this, drawingType, pageAnchoredComponents, attributes, uri, localName, qName);
    	}
    	if(qName.equals("text:section")) {
    		if(sectionStack==null) {
    			sectionStack = new Stack<SectionStartMarker>();
    		}
    		final SectionStartMarker sectionStartMarker = new SectionStartMarker(attributes);
    		sectionStack.push(sectionStartMarker);
    		body.getContent().add(sectionStartMarker);
    		return this;
    	}
		final ElementNSWriter element = new ElementNSWriter(getFileDom(), attributes, uri, qName);
		body.getContent().add(element);
		return new UnknownContentHandler(this, element);
    }

	@Override
	public void endElement(String localName, String qName) throws SAXException {
		super.endElement(localName, qName);

		if(qName.equals("text:section")) {
			final SectionStartMarker sectionStartMarker = sectionStack.pop();
			final SectionEndMarker sectionEndMarker = new SectionEndMarker(sectionStartMarker);
			body.getContent().add(sectionEndMarker);
			sectionStartMarker.setEndMarker(sectionEndMarker);
		}
	}

	@Override
	public void endContext(String qName, String characters) {
		super.endContext(qName, characters);

		if(qName.equals("office:text")) {
			if(!pageAnchoredComponents.isEmpty()) {
				// move these objects into the first paragraph...
				final ParagraphComponent paragraphComponent = getFirstParagraphComponent(new RootComponent(body).getChildComponent(0));
				Paragraph paragraph;
				if(paragraphComponent!=null) {
					paragraph = (Paragraph)paragraphComponent.getObject();
				}
				else {
					paragraph = new Paragraph(null);
					body.getContent().add(paragraph);
				}
				final TextSpan textSpan = new TextSpan();
				paragraph.getContent().addFirst(textSpan);
				textSpan.setContent(pageAnchoredComponents);
			}
		}
	}

	private ParagraphComponent getFirstParagraphComponent(Component component) {
		while(component!=null) {
			if(component instanceof ParagraphComponent) {
				return (ParagraphComponent)component;
			}
			final Component childParagraphComponent = component.getChildComponent(0);
			if(childParagraphComponent!=null) {
				return getFirstParagraphComponent(childParagraphComponent);
			}
			component = component.getNextComponent();
		}
		return null;
	}
}
